/*!
FullCalendar v5.3.2
Docs & License: https://fullcalendar.io/
(c) 2020 Adam Shaw
*/
import { __assign } from 'tslib';
import { rrulestr, RRule } from 'rrule';
import { identity, createDuration, createPlugin, parseMarker } from '@fullcalendar/common';

var RRULE_EVENT_REFINERS = {
    rrule: identity,
    duration: createDuration,
};

var recurring = {
    parse: function (refined, dateEnv) {
        if (refined.rrule != null) {
            var parsed = parseRRule(refined.rrule, dateEnv);
            if (parsed) {
                return {
                    typeData: { rrule: parsed.rrule, isTimeZoneSpecified: parsed.isTimeZoneSpecified },
                    allDayGuess: !parsed.isTimeSpecified,
                    duration: refined.duration,
                };
            }
        }
        return null;
    },
    expand: function (parsed, framingRange, dateEnv) {
        var dates = parsed.rrule.between(framingRange.start, framingRange.end, true); // always UTC
        if (parsed.isTimeZoneSpecified) { // not already in DateMarker form
            dates = dates.map(function (date) { return dateEnv.createMarker(date); });
        }
        // we WANT an inclusive start and in exclusive end, but the js rrule lib will only do either BOTH
        // inclusive or BOTH exclusive, which is stupid: https://github.com/jakubroztocil/rrule/issues/84
        // Workaround: make inclusive, which will generate extra occurences, and then trim.
        dates = dates.filter(function (date) { return date.valueOf() < framingRange.end.valueOf(); });
        return dates;
    },
};
var main = createPlugin({
    recurringTypes: [recurring],
    eventRefiners: RRULE_EVENT_REFINERS,
});
function parseRRule(input, dateEnv) {
    var isTimeSpecified = false;
    var isTimeZoneSpecified = false;
    var rrule;
    if (typeof input === 'string') {
        rrule = rrulestr(input);
        var result = analyzeRRuleString(input);
        isTimeSpecified = result.isTimeSpecified;
        isTimeZoneSpecified = result.isTimeZoneSpecified;
    }
    else if (typeof input === 'object' && input) { // non-null object
        var refined = __assign({}, input); // copy
        if (typeof refined.dtstart === 'string') {
            var result = parseMarker(refined.dtstart);
            // TODO: not DRY
            if (result) {
                isTimeSpecified = isTimeSpecified || !result.isTimeUnspecified;
                isTimeZoneSpecified = isTimeZoneSpecified || result.timeZoneOffset !== null;
                refined.dtstart = new Date(result.marker.valueOf() - (result.timeZoneOffset || 0) * 60 * 1000);
            }
            else { // invalid
                delete refined.dtstart; // best idea?
            }
        }
        if (typeof refined.until === 'string') {
            var result = parseMarker(refined.until);
            // TODO: not DRY
            if (result) {
                isTimeSpecified = isTimeSpecified || !result.isTimeUnspecified;
                isTimeZoneSpecified = isTimeZoneSpecified || result.timeZoneOffset !== null;
                refined.until = new Date(result.marker.valueOf() - (result.timeZoneOffset || 0) * 60 * 1000);
            }
            else { // invalid
                delete refined.until; // best idea?
            }
        }
        if (refined.freq != null) {
            refined.freq = convertConstant(refined.freq);
        }
        if (refined.wkst != null) {
            refined.wkst = convertConstant(refined.wkst);
        }
        else {
            refined.wkst = (dateEnv.weekDow - 1 + 7) % 7; // convert Sunday-first to Monday-first
        }
        if (refined.byweekday != null) {
            refined.byweekday = convertConstants(refined.byweekday); // the plural version
        }
        rrule = new RRule(refined);
    }
    if (rrule) {
        return { rrule: rrule, isTimeSpecified: isTimeSpecified, isTimeZoneSpecified: isTimeZoneSpecified };
    }
    return null;
}
function analyzeRRuleString(str) {
    var isTimeSpecified = false;
    var isTimeZoneSpecified = false;
    function process(whole, introPart, datePart) {
        // TODO: not DRY
        var result = parseMarker(datePart);
        isTimeSpecified = isTimeSpecified || !result.isTimeUnspecified;
        isTimeZoneSpecified = isTimeZoneSpecified || result.timeZoneOffset !== null;
    }
    str.replace(/\b(DTSTART:)([^\n]*)/, process);
    str.replace(/\b(EXDATE:)([^\n]*)/, process);
    str.replace(/\b(UNTIL=)([^;]*)/, process);
    return { isTimeSpecified: isTimeSpecified, isTimeZoneSpecified: isTimeZoneSpecified };
}
function convertConstants(input) {
    if (Array.isArray(input)) {
        return input.map(convertConstant);
    }
    return convertConstant(input);
}
function convertConstant(input) {
    if (typeof input === 'string') {
        return RRule[input.toUpperCase()];
    }
    return input;
}

export default main;
//# sourceMappingURL=main.js.map
